﻿using System.Collections.Generic;
using System.Linq;
using HeuristicLab.Core.Views;
using HeuristicLab.Encodings.SymbolicExpressionTreeEncoding;
using HeuristicLab.Encodings.SymbolicExpressionTreeEncoding.Views;
using HeuristicLab.EvolutionTracking;
using HeuristicLab.MainForm;

namespace HeuristicLab.Problems.DataAnalysis.Symbolic.Views {
  [View("FragmentGraphView")]
  [Content(typeof(IGenealogyGraph<IFragment<ISymbolicExpressionTreeNode>>), IsDefaultView = true)]
  public sealed partial class FragmentGraphView : ItemView {
    private ReingoldTilfordLayoutEngine<TileLayoutNode> layoutEngine;
    private ReingoldTilfordLayoutEngine<ISymbolicExpressionTreeNode> symbolicExpressionEngine;

    private Dictionary<IGenealogyGraphNode<IFragment<ISymbolicExpressionTreeNode>>, TileLayoutNode> tileDictionary;

    private SymbolicExpressionTreeTile Root { get; set; }

    public new IGenealogyGraph<IFragment<ISymbolicExpressionTreeNode>> Content {
      get { return (IGenealogyGraph<IFragment<ISymbolicExpressionTreeNode>>)base.Content; }
      set {
        base.Content = value;
        MakeTiles();
        Draw();
      }
    }

    public FragmentGraphView() {
      InitializeComponent();

      layoutEngine = new ReingoldTilfordLayoutEngine<TileLayoutNode>(n => n.Children);
      symbolicExpressionEngine = new ReingoldTilfordLayoutEngine<ISymbolicExpressionTreeNode>(n => n.Subtrees);
      tileDictionary = new Dictionary<IGenealogyGraphNode<IFragment<ISymbolicExpressionTreeNode>>, TileLayoutNode>();
    }

    private void MakeTiles() {
      var chart = symbolicExpressionChartControl.Chart;
      tileDictionary.Clear();
      foreach (var node in Content.Nodes.Cast<IGenealogyGraphNode<IFragment<ISymbolicExpressionTreeNode>>>()) {
        var tile = new SymbolicExpressionTreeTile(chart);
        tile.LayoutEngine = symbolicExpressionEngine;
        tile.Root = node.Content.Root;
        var tileNode = new TileLayoutNode {
          Tile = tile
        };
        tileDictionary.Add(node, tileNode);
      }

      foreach (var node in Content.Nodes.Where(n => n.InArcs != null && n.InArcs.Count > 0)) {
        var graphNode = (IGenealogyGraphNode<IFragment<ISymbolicExpressionTreeNode>>)node;
        var layoutNode = tileDictionary[graphNode];
        layoutNode.Children = new List<TileLayoutNode>();
        foreach (var c in node.InArcs.Select(x => x.Source as IGenealogyGraphNode<IFragment<ISymbolicExpressionTreeNode>>)) {
          layoutNode.Children.Add(tileDictionary[c]);
        }
      }
    }

    private void Draw() {
      var root = tileDictionary[(IGenealogyGraphNode<IFragment<ISymbolicExpressionTreeNode>>)Content.Nodes[0]];
      var visualNodes = layoutEngine.CalculateLayout(root);

      symbolicExpressionChartControl.UpdateEnabled = false;
      foreach (var visualNode in visualNodes) {
        var tile = visualNode.Content.Tile;
        symbolicExpressionChartControl.Add(tile);
      }
      symbolicExpressionChartControl.UpdateEnabled = true;
      symbolicExpressionChartControl.EnforceUpdate();
    }

    protected override void DeregisterContentEvents() {
      // TODO: Deregister your event handlers here
      base.DeregisterContentEvents();
    }

    protected override void RegisterContentEvents() {
      base.RegisterContentEvents();
      // TODO: Register your event handlers here
    }

    #region Event Handlers (Content)

    // TODO: Put event handlers of the content here

    #endregion

    protected override void OnContentChanged() {
      base.OnContentChanged();
      if (Content == null) {
        // TODO: Add code when content has been changed and is null
      } else {
        // TODO: Add code when content has been changed and is not null
      }
    }


    protected override void SetEnabledStateOfControls() {
      base.SetEnabledStateOfControls();
      // TODO: Enable or disable controls based on whether the content is null or the view is set readonly
    }

    #region Event Handlers (child controls)

    // TODO: Put event handlers of child controls here.

    #endregion
  }

  internal class TileLayoutNode {
    public SymbolicExpressionTreeTile Tile { get; set; }
    public List<TileLayoutNode> Children { get; set; }
  }
}